# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/nc-to-thredds/source/writeNetCDF.py $
# Last committed:     $Revision: 420912 $; $Author: vvuik $; $Date: 2019-11-15 17:12:50 +0100 (vr, 15 nov 2019) $
# -*- coding: utf-8 -*-
"""
Created on Wed Sep 11 14:38:50 2019

@author: hove
"""

import netCDF4
import os
import os.path
import numpy as np
from datetime import datetime
from listFunctions import listFunctions

#%%

def fews_params(ncpar):
    
    return ncpar.replace('_', '.')

#%%
    
def write_location_data(filename, X, Y, lon, lat):
    
    print("Start writing file {}".format(filename))
    if os.path.exists(filename):
        print("File {} already exist. It will be overwritten.".format(filename))
        os.remove(filename)
        
    if not os.path.exists(os.path.dirname(filename)):
        os.makedirs(os.path.dirname(filename))
        
    nc = netCDF4.Dataset(filename, mode='w')

    ### Add geodata ###

    # 1) Latitude

    nc.createDimension('location', 1)
    nc.createVariable('latitude', 'f4', (u'location',))
    # data
    nc.variables['latitude'][:] = lat
    # attributes
    nc.variables['latitude'].long_name = 'measurement location latitude'
    nc.variables['latitude'].standard_name = 'latitude'
    nc.variables['latitude'].units = 'degrees_north'
    nc.variables['latitude'].grid_mapping = 'wgs84'
    nc.variables['latitude'].axis = 'Y'

    # 2) Longitude

    nc.createVariable('longitude', 'f4', (u'location',))
    # data
    nc.variables['longitude'][:] = lon
    # attributes
    nc.variables['longitude'].long_name = 'measurement location longitude'
    nc.variables['longitude'].standard_name = 'longitude'
    nc.variables['longitude'].units = 'degrees_east'
    nc.variables['longitude'].grid_mapping = 'wgs84'
    nc.variables['longitude'].axis = 'X'

    # 3) X

    nc.createVariable('X', 'f4', (u'location',))
    # data
    nc.variables['X'][:] = X
    # attributes
    nc.variables['X'].long_name = 'measurement location x'
    nc.variables['X'].standard_name = 'projection_x_coordinate'
    nc.variables['X'].units = 'm'
    nc.variables['X'].grid_mapping = 'epsg'

    # 4) Y

    nc.createVariable('Y', 'f4', (u'location',))
    # data
    nc.variables['Y'][:] = Y
    # attributes
    nc.variables['Y'].long_name = 'measurement location y'
    nc.variables['Y'].standard_name = 'projection_y_coordinate'
    nc.variables['Y'].units = 'm'
    nc.variables['Y'].grid_mapping = 'epsg'

    # 5) WGS84

    nc.createVariable('wgs84', 'i4')
    # data
    nc.variables['wgs84'][:] = 4326
    # attributes
    nc.variables['wgs84'].standard_name = 'WGS 84'
    nc.variables['wgs84'].epsg = 4326.0
    nc.variables['wgs84'].grid_mapping_name = 'latitude_longitude'
    nc.variables['wgs84'].semi_major_axis = 6.37814e+06
    nc.variables['wgs84'].semi_minor_axis = 6.35675e+06
    nc.variables['wgs84'].inverse_flattening = 298.257
    nc.variables['wgs84'].proj4_params = "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs"
    nc.variables['wgs84'].EPSG_code = 'EPSG:4326'
    nc.variables['wgs84'].projection_name = 'Latitude Longitude'
    nc.variables['wgs84'].comment = 'value is equal to EPSG code'

    # 6) EPSG

    nc.createVariable('epsg', 'i4')
    # data
    nc.variables['epsg'][:] = 7415
    # attributes
    nc.variables['epsg'].standard_name = 'Amersfoort / RD New'
    nc.variables['epsg'].epsg = 7415.0
    nc.variables['epsg'].epsg_name = 'Oblique Stereographic'
    nc.variables['epsg'].grid_mapping_name = 'stereographic'
    nc.variables['epsg'].semi_major_axis = 6.3774e+06
    nc.variables['epsg'].semi_minor_axis = 6.35608e+06
    nc.variables['epsg'].inverse_flattening = 299.153
    nc.variables['epsg'].latitude_of_projection_origin = 52.0922
    nc.variables['epsg'].longitude_of_projection_origin = 5.23155
    nc.variables['epsg'].false_easting = 155000
    nc.variables['epsg'].false_northing = 463000
    nc.variables['epsg'].scale_factor_at_projection_origin = 0.999908
    nc.variables['epsg'].proj4_params = "+proj=sterea +lat_0=52.15616055555555 +lon_0=5.38763888888889 +k=0.999908 +x_0=155000 +y_0=463000 +ellps=bessel +units=m +towgs84=565.4174,50.3319,465.5542,-0.398957388243134,0.343987817378283,-1.87740163998045,4.0725 +no_defs"
    nc.variables['epsg'].EPSG_code = 'EPSG:7415'
    nc.variables['epsg'].projection_name = 'Amersfoort / RD New'
    nc.variables['epsg'].comment = 'value is equal to EPSG code'

    return nc
        
#%% 
        
def write_global_data(nc, instrument, location_id, location_name, startTime, endTime, Z, bedlevel, tZ, SN, drukoffset, scriptdir='./'):
    
    ### Add global info ###

    nc.time_start = datetime.strftime(startTime, format='%Y-%m-%d %H:%M:%S.%f')
    nc.time_end = datetime.strftime(endTime, format='%Y-%m-%d %H:%M:%S.%f')
    nc.time_duration = str(endTime-startTime)
    nc.sensor = instrument
    nc.sensor_serialnumber = SN
    nc.metadata_timestamp = tZ
    nc.platform_id = location_id
    nc.platform_name = location_name
    nc.bed_level = str(bedlevel) + ' m+NAP'
    if instrument=='ADV' or instrument=='AQD':
        nc.drukoffset = str(drukoffset) + 'hPa'

    strFunsRevs = listFunctions(instrument,scriptdir) # call function listFunctions
    nc.functions_used = strFunsRevs
    
    nc.publisher_name = 'HKV Consultants, Tauw, Iv-Infra'
    nc.publisher_url = 'https://www.hkv.nl'
    nc.publisher_institution = 'HKV Consultants'
    nc.publisher_email = 'info@hkv.nl'
    nc.ncfile_created = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    
    # Z 

    nc.createVariable('Z', 'f4', (u'location',))
    # data
    nc.variables['Z'][:] = Z
    # attributes
    if instrument=='ADV':
        nc.variables['Z'].long_name = 'height of sampling volume for velocities'
    elif instrument=='AQD':
        nc.variables['Z'].long_name = 'height of transducers and pressure sensor'
    elif instrument=='ADC':
        nc.variables['Z'].long_name = 'height of transducers'
    else:
        nc.variables['Z'].long_name = 'height of the sensor'
        
    nc.variables['Z'].standard_name = 'sensor height'
    nc.variables['Z'].units = 'm+NAP'
    nc.variables['Z'].axis = 'Z'
    
    if instrument=='ADV':
        nc.createVariable('Zpr', 'f4', (u'location',))
        # data
        nc.variables['Zpr'][:] = Z+0.368
        nc.variables['Zpr'].long_name = 'height of pressure sensor ADV, 0.368 m higher than sampling volume'
        nc.variables['Zpr'].standard_name = 'height pressure sensor'
        nc.variables['Zpr'].units = 'm+NAP'
        nc.variables['Zpr'].axis = 'Z'
    elif instrument=='AQD':
        nc.createVariable('Zpr', 'f4', (u'location',))
        # data
        nc.variables['Zpr'][:] = Z
        nc.variables['Zpr'].long_name = 'height of pressure sensor AQD next to transducers'
        nc.variables['Zpr'].standard_name = 'height pressure sensor'
        nc.variables['Zpr'].units = 'm+NAP'
        nc.variables['Zpr'].axis = 'Z'        
    
    return nc

#%%
        
def write_standard_data(nc, df, parameters, aquo):
        
    ### Add time ###

    nc.createDimension('time', len(df.index))
    nc.createVariable('time', 'f8', (u'time',))
    # data
    nc.variables['time'][:] = np.float64((df.index.astype(np.int64) // 10**6).values)
    # add attributes
    nc.variables['time'].short_name = 'time'
    nc.variables['time'].long_name = 'time of measurement'
    nc.variables['time'].units = 'milliseconds since 1970-01-01 00:00:00.000'
    nc.variables['time'].calendar = 'gregorian'
    
    for ix in parameters:

        nc.createVariable(aquo[ix]['grootheid'], aquo[ix]['datatype'], (u'time'))

        # add attributes from dictionary based on aquo standard and from FEWS data
        nc.variables[aquo[ix]['grootheid']].units = aquo[ix]['units']
        nc.variables[aquo[ix]['grootheid']].dutch_short_name = aquo[ix]['omschrijving']
        nc.variables[aquo[ix]['grootheid']].dutch_description = aquo[ix]['definitie']
        nc.variables[aquo[ix]['grootheid']].short_name = aquo[ix]['description']
        nc.variables[aquo[ix]['grootheid']].description = aquo[ix]['definition']

        nc.variables[aquo[ix]['grootheid']][:] = df[fews_params(ix)].values*aquo[ix]['multiplier']
    
    return nc
            
#%%
            
def write_standard_data_levels(nc, df, levels_dict, aquo_levels, levels):
        
    # 7) Levels

    nc.createDimension('layers', len(levels))

    nc.createVariable('layer_ID', 'f4', (u'layers',))
    # data
    nc.variables['layer_ID'][:] = np.arange(1, len(levels)+1, 1)
    # attributes
    nc.variables['layer_ID'].long_name = 'layer number of measurement (1 is bottom layer)'
    nc.variables['layer_ID'].standard_name = 'layer number'
    nc.variables['layer_ID'].units = '-'
    nc.variables['layer_ID'].axis = 'Z'

    # 8) Levels m+NAP

    nc.createVariable('layer_level', 'f4', (u'layers',))
    # data
    nc.variables['layer_level'][:] = levels
    # attributes
    nc.variables['layer_level'].long_name = 'bottom level of measurement layer'
    nc.variables['layer_level'].standard_name = 'layer level'
    nc.variables['layer_level'].units = 'm+NAP'
    nc.variables['layer_level'].axis = 'Z'
    
    # parameters with levels
    for ix in levels_dict.keys():

        nc.createVariable(aquo_levels[ix]['grootheid'], aquo_levels[ix]['datatype'], (u'time', u'layers'))
        nc.variables[aquo_levels[ix]['grootheid']].units = aquo_levels[ix]['units']
        nc.variables[aquo_levels[ix]['grootheid']].dutch_short_name = aquo_levels[ix]['omschrijving']
        nc.variables[aquo_levels[ix]['grootheid']].dutch_description = aquo_levels[ix]['definitie']
        nc.variables[aquo_levels[ix]['grootheid']].short_name = aquo_levels[ix]['description']
        nc.variables[aquo_levels[ix]['grootheid']].description = aquo_levels[ix]['definition']
        
        nc.variables[aquo_levels[ix]['grootheid']][:] = df[[fews_params(w) for w in levels_dict[ix]]].values*aquo_levels[ix]['multiplier']
        
    return nc

#%%
    
def write_tailored_data(nc, spectra, li, frequency, parameters_tailored, aquo, theta=None, S_total=None):
    
    ### Add time ###

    nc.createDimension('time', len(li.index))
    nc.createVariable('time', 'f8', (u'time',))
    # data
    nc.variables['time'][:] = np.float64((li.index.astype(np.int64) // 10**6).values)
    # add attributes
    nc.variables['time'].short_name = 'time'
    nc.variables['time'].long_name = 'time of measurement'
    nc.variables['time'].units = 'milliseconds since 1970-01-01 00:00:00.000'
    nc.variables['time'].calendar = 'gregorian'
    
    ### Add frequency ###
        
    nc.createDimension('frequency', len(frequency))
    nc.createVariable('frequency', 'f4', (u'frequency',))
    # add data
    nc.variables['frequency'][:] = frequency
    # add attributes
    nc.variables['frequency'].short_name = 'frequency'
    nc.variables['frequency'].long_name = 'frequency from 0 Hz to 2 Hz'
    nc.variables['frequency'].units = 'Hz'
    
    try:
        ### Add wave direction ###
                
        nc.createDimension('wave_direction', len(theta))
        nc.createVariable('wave_direction', 'f4', (u'wave_direction',))
        # add data
        nc.variables['wave_direction'][:] = theta
        # add attributes
        nc.variables['wave_direction'].short_name = 'wave direction (cartesian coordinate system)'
        nc.variables['wave_direction'].long_name = 'wave direction (cartesian coordinate system)'
        nc.variables['wave_direction'].units = 'rad'
    except:
        pass
    
    for ix in parameters_tailored:

        if ix == 'vy':
            nc.createVariable(aquo[ix]['grootheid'], aquo[ix]['datatype'], (u'time', u'frequency'))
            nc.variables[aquo[ix]['grootheid']].units = aquo[ix]['units']  
            nc.variables[aquo[ix]['grootheid']].dutch_short_name = aquo[ix]['omschrijving']
            nc.variables[aquo[ix]['grootheid']].dutch_description = aquo[ix]['definitie']
            nc.variables[aquo[ix]['grootheid']].short_name = aquo[ix]['description']
            nc.variables[aquo[ix]['grootheid']].description = aquo[ix]['definition']
            nc.variables[aquo[ix]['grootheid']][:] = spectra
        elif ix == 'S':
            nc.createVariable(aquo[ix]['grootheid'], aquo[ix]['datatype'], (u'time', u'frequency', u'wave_direction'))
            nc.variables[aquo[ix]['grootheid']].units = aquo[ix]['units']  
            nc.variables[aquo[ix]['grootheid']].dutch_short_name = aquo[ix]['omschrijving']
            nc.variables[aquo[ix]['grootheid']].dutch_description = aquo[ix]['definitie']
            nc.variables[aquo[ix]['grootheid']].short_name = aquo[ix]['description']
            nc.variables[aquo[ix]['grootheid']].description = aquo[ix]['definition']
            nc.variables[aquo[ix]['grootheid']][:] = S_total
        else:
            nc.createVariable(aquo[ix]['grootheid'], aquo[ix]['datatype'], (u'time'))
            nc.variables[aquo[ix]['grootheid']].units = aquo[ix]['units']  
            nc.variables[aquo[ix]['grootheid']].dutch_short_name = aquo[ix]['omschrijving']
            nc.variables[aquo[ix]['grootheid']].dutch_description = aquo[ix]['definitie']
            nc.variables[aquo[ix]['grootheid']].short_name = aquo[ix]['description']
            nc.variables[aquo[ix]['grootheid']].description = aquo[ix]['definition']
            nc.variables[aquo[ix]['grootheid']][:] = li[ix].values
    
    return nc    